import java.util.*;

/**
 * PolynomialADT
 * Defines the ADT for a polynomial.
 * modified by edhong, 1/11/06.
 * @author TCSS 342
 * @version 1.0
 */
public interface PolynomialADT {


    /**
     * Add a term to the polynomial.
     * If a term with the same exponent is already in the polynomial,
     * then add the coefficients.
     * @param coefficient  coefficient of the new term
     * @param exponent     exponent of the new term
     */
    public void insertTerm(int coefficient, int exponent);


    /**
     * Set the polynomial to the zero polynomial.
     */
    public void zeroPolynomial();

    /**
     * Compute the negation of the polynomial.
     * @return  the negation of the polynomial.
     */
    public PolynomialADT negate() ;

    /**
     * Add a polynomial to the polynomial.
     * @param rhs  the other polynomial to add
     * @return  (this + rhs)
     */
    public PolynomialADT plus(PolynomialADT rhs);

    /**
     * Subtract a polynomial from the polynomial.
     * @param rhs  the other polynomial
     * @return  (this - rhs)
     */
    public PolynomialADT minus(PolynomialADT rhs) ;

    /**
     * Multiply a polynomial with the polynomial.
     * If this has n terms and rhs has m terms, then
     * this routine runs in O(nm) time.
     * @param rhs  the other polynomial
     * @return  (this * rhs)
     */
    public PolynomialADT times(PolynomialADT rhs) ;


    /**
     * Compute the derivative of the polynomial.
     * @return  the derivative of the polynomial.
     */
    public PolynomialADT derivative() ;

    /**
     * Return a String representation of the polynomial.
     * (e.g., -3x^2 + 2x + 1 - x^(-2) )
     * @return  the String that represents the polynomial
     */
    public String toString();
}